;*******************************************************************************
;*
;*      Titel:  LCD-Bibliothek: Unterprogramme zur Ansteuerung eines 2-zeiligen
;*              LC-Displays (16 Zeichen/Zeile, Controller-Typ HD44780) mit 4 Bit
;*              (SR_LCD_2x16_4Bit.asm)
;*
;*      Datum:  03/04/10        Version:        1.1 (06/05/12)
;*      Autor:  WEIGU
;*
;*      Informationen zur Beschaltung:
;*
;*      Prozessor:   ATmega32       Quarzfrequenz: 16MHz
;*      Ausgaenge:   4 Datenleitungen D4-D7, RS(Function Select)
;*                   E (Taktleitung), R//W an Masse
;*      Es muss eine Zeitschleifenbibliothek mit den Unterprogrammen W100us,
;*      W500us, W5ms und W100ms eingebunden werden
;*      
;*      Unterprogramme:
;*      LCDINI:    Initialisiert das LCD-Interface für 4 Bit
;*      LCDCLK:    Taktimpuls senden
;*      LCDSCO:    Sendet einen Befehl zum LCD
;*      LCDSDA:    Sendet ein Zeichen zum LCD
;*      LCDSTF:    Sendet einen Text-String aus dem Flash zum LCD
;*      LCDSTR:    Sendet einen Text-String aus dem SRAM zum LCD
;*      LCDCLR:    Loescht das Display
;*      DEFCHR:    Definiert ein Anwender-Sonderzeichen
;*
;*      Copyright (c) 2012   Guy WEILER         weigu[at]weigu[dot]lu
;*
;*      Hiermit wird unentgeltlich, jeder Person, die eine Kopie der Software
;*      und der zugehoerigen Dokumentationen (die "Software") erhaelt, die
;*      Erlaubnis erteilt, uneingeschraenkt zu benutzen, inklusive und ohne
;*      Ausnahme, dem Recht, sie zu verwenden, kopieren, aendern, fusionieren,
;*      verlegen, verbreiten, unterlizenzieren und/oder zu verkaufen, und
;*      Personen, die diese Software erhalten, diese Rechte zu geben, unter den
;*      folgenden Bedingungen:
;*
;*      ;Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in alle
;*      ;Kopien oder Teilkopien der Software beizulegen.
;*      ;
;*      ;DIE SOFTWARE WIRD OHNE JEDE AUSDRUECKLICHE ODER IMPLIZIERTE GARANTIE
;*      ;BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FUER DEN
;*      ;VORGESEHENEN ODER EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTS-
;*      ;VERLETZUNG, JEDOCH NICHT DARAUF BESCHRAENKT. IN KEINEM FALL SIND DIE
;*      ;AUTOREN ODER COPYRIGHTINHABER FUER JEGLICHEN SCHADEN ODER SONSTIGE
;*      ;ANSPRUECHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFUELLUNG EINES
;*      ;VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER SOFTWARE
;*      ;ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
;*
;*******************************************************************************

;Damit der Code im Flash gering bleibt werden ausschliesslich die benutzten
;Unterprogramme freigeschaltet (Flag = 1)
.EQU    Flag_LCDINI   = 1	;Initialisiert das LCD-Interface für 4 Bit
.EQU    Flag_LCDCLK   = 1	;Taktimpuls sendent
.EQU    Flag_LCDSCO   = 1	;Sendet einen Befehl zum LCD
.EQU    Flag_LCDSDA   = 1	;Sendet ein Zeichen zum LCD
.EQU    Flag_LCDSTF   = 1	;Sendet einen Text-String aus dem Flash zum LCD
.EQU    Flag_LCDSTR   = 1	;Sendet einen Text-String aus dem SRAM zum LCD
.EQU    Flag_LCDCLR   = 1	;Loescht das Display
.EQU    Flag_LCDDEF   = 0	;Definiert ein Anwender-Sonderzeichen

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Zuweisungen
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EQU	LCDDPORT = PORTB	;Datenport fuer das Display
.EQU	LCDDDDR = DDRB
.EQU	LCDCPORT = PORTD	;Steuerport fuer das Display
.EQU	LCDCDDR = DDRD
.EQU    RS 	= 0		;Register Select (0 = Befehl, 1 = Daten)
.EQU    ENA 	= 1             ;Taktleitung
.EQU	LowNib 	= 0		;LowNib = 1: D4-D7 an Pin0-Pin3
				;LowNib = 0  D4-D7 an Pin4-Pin7
				
.EQU	PL1 = 0x80		;Adresszeiger fuer die erste Zeile
.EQU	PL2 = 0xC0		;Adresszeiger fuer die zweite Zeile
.EQU	PT1 = PL1+2		;Adresszeiger fuer den ersten Text
.EQU	PT2 = PL1+9		;Adresszeiger fuer den zweiten Text
.EQU	PT3 = PL2+5		;Adresszeiger fuer den dritten Text
.EQU	PT4 = PL1+9		;Adresszeiger fuer den vierten Text
;-------------------------------------------------------------------------------
;	Initialisierung des HD44780 mit 4-Bit-Schnittstelle
;-------------------------------------------------------------------------------
.IF Flag_LCDINI == 1
LCDINI: push    r16
	push    r18
	
        in      r16,LCDDDDR      ;6 Ausgaenge initialisieren
        .IF LowNib == 1
        ori     r16,0x0F
        .ELSE
        ori     r16,0xF0
        .ENDIF
        out     LCDDDDR,r16
        sbi	LCDCDDR,RS
        sbi	LCDCDDR,ENA
        rcall	W100ms          ;Warte bis Diplaycontroller hochgefahren ist
        cbi	LCDCPORT,RS       

        ;zuerst in den 8-Bit Modus um nach Reset keinen Bloedsinn anzuzeigen        
	ldi	r18,3        
LCDINL: in      r16,LCDDPORT    ;Befehl 0x30: Initialisierung 8 Bit (3x) und 5ms
        .IF LowNib == 1		;warten 
        ori 	r16,0x03	;(Nibble verdreht, siehe Anschluss)
        andi    r16,0xF3
        .ELSE
        ori 	r16,0x30
        andi    r16,0x3F
        .ENDIF
        out     LCDDPORT,r16     
	rcall	LCDCLK          ;Taktimpuls senden        
        rcall   W5ms
	dec	r18
	brne	LCDINL

        ;dann in den 4 Bit-Modus
	in      r16,LCDDPORT     ;Befehl 0x20: Initialisierung 4 Bit	
	.IF LowNib == 1		;warten 
        ori 	r16,0x02	;(Nibble verdreht, siehe Anschluss)
        andi    r16,0xF2
        .ELSE
        ori 	r16,0x20
        andi    r16,0x2F
        .ENDIF
        out     LCDDPORT,r16
        rcall   LCDCLK          ;Taktimpuls senden

	ldi	r24,0x28	;Befehl 0x28: 4 Bit Daten, 2 Zeilen, Matrix 5x8
	rcall	LCDSCO		;

	ldi	r24,0x0F	;0x0C: Display, Cursor, Blinken eingeschaltet
	rcall	LCDSCO		;

	ldi	r24,0x06	;0x06: Eingabemodus = Adresse inkrem., kein Shift
	rcall	LCDSCO		;Befehl senden

        ldi     r24,0x0C        ;0x0C: Display on, Cursor off, Blinken off
        rcall   LCDSCO
        
	pop	r18
        pop     r16
	ret
.ENDIF

;-------------------------------------------------------------------------------
;       Taktimpuls senden
;-------------------------------------------------------------------------------
.IF Flag_LCDCLK == 1
LCDCLK: sbi     LCDCPORT,ENA       ;Taktsignal (High Impuls senden)
        rcall   W100us
        cbi     LCDCPORT,ENA
        rcall   W500us
        ret
.ENDIF
;-------------------------------------------------------------------------------
;	Befehl zum LCD senden (RS = 0 fuer Befehl, Befehl in r24)
;-------------------------------------------------------------------------------
.IF Flag_LCDSCO == 1
LCDSCO: push    r16
        push    r17
        push    r24

        cbi     LCDCPORT,RS	;Register Select = 0 fuer Befehl
	mov     r16,r24         ;HighNibble ausmaskieren und ausgeben ohne
	andi    r16,0xF0        ;andere Bits zu beeinflussen
	.IF LowNib == 1
        swap    r16             
        .ENDIF        
        in      r17,LCDDPORT
        or      r17,r16
        .IF LowNib == 1		
        ori     r16,0xF0
        .ELSE
        ori     r16,0x0F
        .ENDIF                
        and     r17,r16   
        out     LCDDPORT,r17
        rcall   LCDCLK          ;Taktimpuls senden        
        andi    r24,0x0F        ;LowNibble ausmaskieren und ausgeben ohne
        .IF LowNib == 0		
        swap    r24             
        .ENDIF        
        in      r17,LCDDPORT    ;andere Bits zu beeinflussen
        or      r17,r24
        .IF LowNib == 1		
        ori     r24,0xF0
        .ELSE
        ori     r24,0x0F
        .ENDIF                
        and     r17,r24        
        out     LCDDPORT,r17
        rcall   LCDCLK          ;Taktimpuls senden

	pop	r24
	pop     r17
        pop     r16
        ret
.ENDIF

;-------------------------------------------------------------------------------
;	Datenbyte zum LCD senden (RS = 1 fuer Daten, Daten in r24)
;-------------------------------------------------------------------------------
.IF Flag_LCDSDA == 1
LCDSDA: push    r16
        push    r17
        push    r24

        sbi     LCDCPORT,RS	;Register Select = 1 fuer Befehl
        mov     r16,r24         ;HighNibble ausmaskieren und ausgeben ohne
	andi    r16,0xF0        ;andere Bits zu beeinflussen
	.IF LowNib == 1		
        swap    r16             
        .ENDIF        
        in      r17,LCDDPORT
        or      r17,r16
        .IF LowNib == 1		
        ori     r16,0xF0
        .ELSE
        ori     r16,0x0F
        .ENDIF                
        and     r17,r16
        out     LCDDPORT,r17
        rcall   LCDCLK          ;Taktimpuls senden        
        andi    r24,0x0F        ;LowNibble ausmaskieren und ausgeben ohne
        .IF LowNib == 0		
        swap    r24             
        .ENDIF        
        in      r17,LCDDPORT    ;andere Bits zu beeinflussen
        or      r17,r24
        .IF LowNib == 1		
        ori     r24,0xF0
        .ELSE
        ori     r24,0x0F
        .ENDIF                
        and     r17,r24
        out     LCDDPORT,r17
        rcall   LCDCLK          ;Taktimpuls senden

        pop	r24
        pop     r17
        pop     r16
        ret
.ENDIF        

;-------------------------------------------------------------------------------
;	TextAusgabe FLASH: Gibt Zeichen an das Display bis Nullbyte
;	Adresse des ersten Zeichens wird mit Z-Pointer übergeben.
;-------------------------------------------------------------------------------
.IF Flag_LCDSTF == 1
LCDSTF: push	r24
	push	ZH
	push	ZL
	
	lpm     r24,Z+
        cpi     r24,0
        breq    LCDSFR
	rcall	LCDSDA		;Zeichen senden, Autoinkrement der Adresse
        rjmp	LCDSTF		;alle Zeichen an das LCD ausgeben
        
LCDSFR:	pop	ZL
	pop	ZH
	pop	r24
	ret
.ENDIF        

;-------------------------------------------------------------------------------
;       TextAusgabe RAM: Gibt Zeichen an das Display bis Nullbyte
;       Adresse des ersten Zeichens wird mit Z-Pointer übergeben.
;-------------------------------------------------------------------------------
.IF Flag_LCDSTR == 1
LCDSTR: push	r24
	push	ZH
	push	ZL

	ld      r24,Z+
        cpi     r24,0
        breq    LCDSRR
        rcall   LCDSDA          ;Zeichen senden, Autoinkrement der Adresse
        rjmp    LCDSTR          ;alle Zeichen an das LCD ausgeben
        
LCDSRR: pop	ZL
	pop	ZH
	pop	r24
 	ret
.ENDIF        

;-------------------------------------------------------------------------------
;       Display loeschen
;-------------------------------------------------------------------------------
.IF Flag_LCDCLR == 1
LCDCLR: push	r24

	ldi     r24,0x01        ;0x01: Display Clear
        rcall   LCDSCO          
        rcall   W5ms
        
        pop	r24
        ret
.ENDIF        

;-------------------------------------------------------------------------------
;       Definition eigener Sonderzeichen (Zeichencode in r24)
;       
;       ueberarbeiten!
;-------------------------------------------------------------------------------
.IF Flag_LCDDEF == 1
DEFCHR: push    r16
        lsl	r24		;r24 mit 2 multipliziert
	lsl	r24		;r24 mit 4 multipliziert
	lsl	r24		;r24 mit 8 multipl.
 			;(8 Zeilen pro Zeichen CG-RAM)
	mov	ZL,r24		;Z-Ptr zeigt auf Anf.-Adr. des Zeichens in der
				;Char.-Tab 
	ldi	ZH,high(CHRTAB);High-Byte der Punktemuster-Tabellenadr.
	rol	ZH		;Z-Ptr um 1 Pos. nach links schieben
	sbr	r24,1<<6	;Bit6 setzen -> Zieladr. ist im CG-RAM
	
	rcall	LCDSCO		;Adressenzähler im CG-RAM setzen
	ldi	r16,8
DC1:	lpm			;Punktemuster nach R0 laden
	mov	r24,R0		;Punktemuster nach r24 kopieren
	
	rcall	LCDSDA		;Zeichenmuster senden, Autoinkrement der Adresse
	adiw	ZL,1		;Z-Ptr inkrementieren
	dec	r16		;Zähler - 1
	brne	DC1		;alle 8 Zeilen ins CG-RAM kopieren
        pop     r16
	ret
.ENDIF        

CHRTAB:
.DB   $04,$04,$1F,$04,$04,$00,$1F,$00   ; Ch.-Code $00: Zeichen "+/-"
.DB   $0C,$02,$04,$08,$0E,$00,$00,$00   ; Ch.-Code $01: Zeichen "^2"
.DB   $0C,$02,$0C,$02,$0C,$00,$00,$00   ; Ch.-Code $02: Zeichen "^3"
.DB   $18,$06,$01,$06,$18,$00,$1F,$00   ; Ch.-Code $03: Zeichen ">="
.DB   $03,$0C,$10,$0C,$03,$00,$1F,$00   ; Ch.-Code $04: Zeichen "<="
.DB   $04,$04,$1F,$04,$1F,$04,$04,$00   ; Ch.-Code $05: Zeichen "ungleich"
.DB   $00,$1F,$00,$1F,$00,$1F,$00,$00   ; Ch.-Code $06: Zeichen "identisch"
.DB   $15,$15,$15,$15,$15,$15,$15,$00   ; Ch.-Code $06: Zeichen "3 Balken"
TEXT1:
.DB   "   :      /  /  ",0,0
TEXT2:
.DB   "ppm ",0,0
TEXT3:
.DB   0,' ',1,' ',2,' ',3,' ',4,' ',5,' ',6,' ',7,' '
TEXT4:
.DB   "  Display-Test ",0
TEXT5:
.DB   "Char 6-8 blinken",0,0
TEXT6:
.DB   "Cursor-Pos. = 65",0,0
TEXT7:
.DB   "Grad C",0,0
