;*******************************************************************************
;*
;*	Titel:	Bibliothek mit Zeitunterprogrammen (SR_TIME_16M.asm)
;*	Datum:	10/04/10	Version:	0.4
;*	Autor:	WEIGU
;*
;*	Informationen zur Beschaltung:
;*	Prozessor:	ATMEGA32	Quarzfrequenz:	 extern 16MHz
;*			
;*	Informationen zur Funktionsweise:
;*
;*	Die Bibliothek enthaelt Zeitschleifenunterprogramme zur Verwendung mit
;*	einem externen 16MHz Quarz.
;*	
;*	
;*	Die Zeitschleifen:
;*
;*	W1us, W10us, W100us, W150us, W500us und W10ms sind sehr genau.
;*
;*	Bei den anderen Zeitschleifen kann der Fehler durch push, pop, ret
;*	und die Schleifenbefehle vernachlaessigt werden.
;*	Weitere feste Zeitschleifen:
;*
;*	W5ms, W100ms, W500ms, W1s, W5s, W10s, W1min und W10min		
;*
;*	Zeitschleifen mit einstellbarer Zeit über das X-Register:		
;*
;*	WX100us, WX1ms, WX10ms, WX100ms, WX1s, WX1min
;*      
;*      
;*      Copyright (c) 2010   Guy WEILER         weigu[at]weigu[dot]lu
;*
;*      Hiermit wird unentgeltlich, jeder Person, die eine Kopie der Software
;*      und der zugehoerigen Dokumentationen (die "Software") erhaelt, die
;*      Erlaubnis erteilt, uneingeschraenkt zu benutzen, inklusive und ohne
;*      Ausnahme, dem Recht, sie zu verwenden, kopieren, aendern, fusionieren,
;*      verlegen, verbreiten, unterlizenzieren und/oder zu verkaufen, und
;*      Personen, die diese Software erhalten, diese Rechte zu geben, unter den
;*      folgenden Bedingungen:
;*
;*      ;Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in alle
;*      ;Kopien oder Teilkopien der Software beizulegen.
;*      ;
;*      ;DIE SOFTWARE WIRD OHNE JEDE AUSDRUECKLICHE ODER IMPLIZIERTE GARANTIE
;*      ;BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FUER DEN
;*      ;VORGESEHENEN ODER EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTS-
;*      ;VERLETZUNG, JEDOCH NICHT DARAUF BESCHRAENKT. IN KEINEM FALL SIND DIE
;*      ;AUTOREN ODER COPYRIGHTINHABER FUER JEGLICHEN SCHADEN ODER SONSTIGE
;*      ;ANSPRUECHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFUELLUNG EINES
;*      ;VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER SOFTWARE
;*      ;ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
;*
;*******************************************************************************
;-------------------------------------------------------------------------------
;       16-Bit-Zeitschleife wartet 1us (Wait 1us)
;       Hierzu werden bei 16MHz 1/0,0625=16 Taktezyklen benötigt.
;	Dies erreichen 9 nop-Befehle und ein ret-Befehl da der rcall Befehle
;       auch 3 Taktzyklen benötigt.
;-------------------------------------------------------------------------------
W1us:	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       16-Bit-Zeitschleife wartet 5us (Wait 5us)
;-------------------------------------------------------------------------------
W5us:	push	WL	;rette verwendete Registerinhalte
	push	WH
	ldi     WL,0x10 ;Initialisiere den Schleifenzaehler W
        ldi     WH,0x00 ;G = (t-tT)/4tT = 5us/4*62,5ns-1/4 = 20-1/4
                        ;Korrektur: -15tT fuer rcall, 2 push, 2 pop, ret
                        ;1 Durchlauf der Schleife benoetigt 4tT
                        ;GKorrektur = G-1/4-15/4 = 20-4 = 16 = 0x0010
W5usL:	sbiw	WL,1	;Dekrementiere den Schleifenzaehler W
	brne	W5usL	;Verlasse die Schleife bei Schleifenzaehler W=0
        pop	WH	;Wiederherstellung der verwendeten Registerinhalte
	pop	WL
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       16-Bit-Zeitschleife wartet 10us (Wait 10us)
;-------------------------------------------------------------------------------
W10us:  push    WL      ;rette verwendete Registerinhalte
        push    WH
        ldi     WL,0x24 ;Initialisiere den Schleifenzaehler W
        ldi     WH,0x00 ;G = (t-tT)/4tT = 10us/4*62,5ns-1/4 = 40-1/4
                        ;Korrektur: -15tT fuer rcall, 2 push, 2 pop, ret
                        ;1 Durchlauf der Schleife benoetigt 4tT
                        ;GKorrektur = G-1/4-15/4 = 40-4 = 36 = 0x0024
W10usL: sbiw    WL,1    ;Dekrementiere den Schleifenzaehler W
        brne    W10usL  ;Verlasse die Schleife bei Schleifenzaehler W=0
        pop     WH      ;Wiederherstellung der verwendeten Registerinhalte
        pop     WL
        ret             ;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       16-Bit-Zeitschleife wartet 50us (Wait 50us)
;-------------------------------------------------------------------------------
W50us:  push    WL      ;rette verwendete Registerinhalte
        push    WH
        ldi     WL,0xC4 ;Initialisiere den Schleifenzaehler W
        ldi     WH,0x00 ;G = (t-tT)/4tT = 50us/4*62,5ns-1/4 = 200-1/4
                        ;Korrektur: -15tT fuer rcall, 2 push, 2 pop, ret
                        ;1 Durchlauf der Schleife benoetigt 4tT
                        ;GKorrektur = G-1/4-15/4 = 200-4 = 196 = 0x00C4
W50usL: sbiw   WL,1     ;Dekrementiere den Schleifenzaehler W
        brne    W100usL ;Verlasse die Schleife bei Schleifenzaehler W=0
        pop     WH      ;Wiederherstellung der verwendeten Registerinhalte
        pop     WL
        ret             ;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       16-Bit-Zeitschleife wartet 100us (Wait 100us)
;-------------------------------------------------------------------------------
W100us:	push	WL	;rette verwendete Registerinhalte
	push	WH
	ldi     WL,0x8C ;Initialisiere den Schleifenzaehler W
        ldi     WH,0x01 ;G = (t-tT)/4tT = 100us/4*62,5ns-1/4 = 400-1/4
                        ;Korrektur: -15tT fuer rcall, 2 push, 2 pop, ret
                        ;1 Durchlauf der Schleife benoetigt 4tT
                        ;GKorrektur = G-1/4-15/4 = 400-4 = 396 = 0x018C
W100usL: sbiw	WL,1	;Dekrementiere den Schleifenzaehler W
	brne	W100usL	;Verlasse die Schleife bei Schleifenzaehler W=0
	pop	WH	;Wiederherstellung der verwendeten Registerinhalte
	pop	WL
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       16-Bit-Zeitschleife wartet 500us (Wait 500us)
;-------------------------------------------------------------------------------
W500us: push    WL      ;rette verwendete Registerinhalte
        push    WH
        ldi     WL,0xCC ;Initialisiere den Schleifenzaehler W
        ldi     WH,0x07 ;G = (t-tT)/4tT = 500us/4*62,5ns-1/4 = 2000-1/4
                        ;Korrektur: -15tT fuer rcall, 2 push, 2 pop, ret
                        ;1 Durchlauf der Schleife benoetigt 4tT
                        ;GKorrektur = G-1/4-15/4 = 2000-4 = 1996 = 0x07CC
W500usL: sbiw   WL,1    ;Dekrementiere den Schleifenzaehler W
        brne    W500usL ;Verlasse die Schleife bei Schleifenzaehler W=0
        pop     WH      ;Wiederherstellung der verwendeten Registerinhalte
        pop     WL
        ret             ;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       16-Bit-Zeitschleife wartet 1ms (Wait 1ms)
;-------------------------------------------------------------------------------
W1ms:	push	WL	;rette verwendete Registerinhalte
	push	WH
        ldi     WL,0x9C ;Initialisiere den Schleifenzaehler W
        ldi     WH,0x0F ;G = (t-tT)/4tT = 1ms/4*62,5ns-1/4 = 4000-1/4
                        ;Korrektur: -15tT fuer rcall, 2 push, 2 pop, ret
                        ;1 Durchlauf der Schleife benoetigt 4tT
                        ;GKorrektur = G-1/4-15/4 = 4000-4 = 3996 = 0x0F9C
                        ;(Korrektur koennte man vernachlaessigen,Fehler 0,1%)
W1msL:	sbiw	WL,1	;Dekrementiere den Schleifenzaehler W
	brne	W1msL	;Verlasse die Schleife bei Schleifenzaehler W=0
	pop	WH	;Wiederherstellung der verwendeten Registerinhalte
	pop	WL
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       16-Bit-Zeitschleife wartet 5ms (Wait 5ms)
;-------------------------------------------------------------------------------
W5ms:   push    WL      ;rette verwendete Registerinhalte
        push    WH
        ldi     WL,0x1C ;Initialisiere den Schleifenzaehler W
        ldi     WH,0x4E ;G = (t-tT)/4tT = 5ms/4*62,5ns-1/4 = 20000-1/4
                        ;Korrektur: -15tT fuer rcall, 2 push, 2 pop, ret
                        ;1 Durchlauf der Schleife benoetigt 4tT
                        ;GKorrektur = G-1/4-15/4 = 20000-4 = 19996 = 0x4E1C
                        ;(Korrektur koennte man vernachlaessigen)
W5msL:  sbiw    WL,1    ;Dekrementiere den Schleifenzaehler W
        brne    W5msL   ;Verlasse die Schleife bei Schleifenzaehler W=0
        pop     WH      ;Wiederherstellung der verwendeten Registerinhalte
        pop     WL
        ret             ;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       16-Bit-Zeitschleife wartet 10ms (Wait 10ms)
;-------------------------------------------------------------------------------
W10ms:	push	WL	;rette verwendete Registerinhalte
	push	WH
	ldi     WL,0x3C ;Initialisiere den Schleifenzaehler W
        ldi     WH,0x9C ;G = (t-tT)/4tT = 10ms/4*62,5ns-1/4 = 40000-1/4
                        ;Korrektur: -15tT fuer rcall, 2 push, 2 pop, ret
                        ;1 Durchlauf der Schleife benoetigt 4tT
                        ;GKorrektur = G-1/4-15/4 = 40000-4 = 39996 = 0x9C3C
                        ;(Korrektur koennte man vernachlaessigen)
W10msL:	sbiw	WL,1	;Dekrementiere den Schleifenzaehler W
	brne	W10msL	;Verlasse die Schleife bei Schleifenzaehler W=0
	pop	WH	;Wiederherstellung der verwendeten Registerinhalte
	pop	WL
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet 100ms (Wait 100ms)
;	Der Fehler durch zusaetz. Befehle kann hier vernachlässigt werden. 
;-------------------------------------------------------------------------------

W100ms:	push	XL	;rette verwendete Registerinhalte
	ldi	XL,0x0A	;Zaehler = 10d
W100msL: rcall	W10ms	;Rufe innere Schleife (Unterprogramm W10ms)
	dec	XL	;Dekrementiere den Schleifenzaehler X
	brne	W100msL	;Verlasse die Schleife bei Schleifenzaehler XL=0
	pop	XL	;Wiederherstellung der verwendeten Registerinhalte
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet 500ms (Wait 500ms)
;	Der Fehler durch zusaetz. Befehle kann hier vernachlässigt werden. 
;-------------------------------------------------------------------------------

W500ms:	push	XL	;rette verwendete Registerinhalte
	ldi	XL,0x32	;Zaehler = 50d
W500msL: rcall	W10ms	;Rufe innere Schleife (Unterprogramm W10ms)
	dec	XL	;Dekrementiere den Schleifenzaehler X
	brne	W500msL	;Verlasse die Schleife bei Schleifenzaehler XL=0
	pop	XL	;Wiederherstellung der verwendeten Registerinhalte
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet 1s (Wait 1s)
;	Der Fehler durch zusaetz. Befehle kann hier vernachlässigt werden. 
;-------------------------------------------------------------------------------

W1s:	push	XL	;rette verwendete Registerinhalte
	ldi	XL,0x64	;Zaehler = 100d
W1sL:	rcall	W10ms	;Rufe innere Schleife (Unterprogramm W10ms)
	dec	XL	;Dekrementiere den Schleifenzaehler X
	brne	W1sL	;Verlasse die Schleife bei Schleifenzaehler XL=0
	pop	XL	;Wiederherstellung der verwendeten Registerinhalte
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet 5s (Wait 5s)
;	Der Fehler durch zusaetz. Befehle kann hier vernachlässigt werden. 
;-------------------------------------------------------------------------------

W5s:	push	XL	;rette verwendete Registerinhalte
	push	XH
	ldi	XL,0xF4	;Zaehler = 500d
	ldi	XH,0x01	;
W5sL:	rcall	W10ms	;Rufe innere Schleife (Unterprogramm W10ms)
	sbiw	XL,1	;Dekrementiere den Schleifenzaehler X
	brne	W5sL	;Verlasse die Schleife bei Schleifenzaehler X=0
	pop	XH	;Wiederherstellung der verwendeten Registerinhalte
	pop	XL	
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet 10s (Wait 10s)
;	Der Fehler durch zusaetz. Befehle kann hier vernachlässigt werden. 
;-------------------------------------------------------------------------------

W10s:	push	XL	;rette verwendete Registerinhalte
	push	XH
	ldi	XL,0xE8	;Zaehler = 1000d
	ldi	XH,0x03	;
W10sL:	rcall	W10ms	;Rufe innere Schleife (Unterprogramm W10ms)
	sbiw	XL,1	;Dekrementiere den Schleifenzaehler X
	brne	W10sL	;Verlasse die Schleife bei Schleifenzaehler X=0
	pop	XH	;Wiederherstellung der verwendeten Registerinhalte
	pop	XL	
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet 1min (Wait 1min)
;	Der Fehler durch zusaetz. Befehle kann hier vernachlässigt werden. 
;-------------------------------------------------------------------------------

W1min:	push	XL	;rette verwendete Registerinhalte
	push	XH
	ldi	XL,0x70	;Zaehler = 6000d
	ldi	XH,0x17	;
W1minL:	rcall	W10ms	;Rufe innere Schleife (Unterprogramm W10ms)
	sbiw	XL,1	;Dekrementiere den Schleifenzaehler X
	brne	W1minL	;Verlasse die Schleife bei Schleifenzaehler X=0
	pop	XH	;Wiederherstellung der verwendeten Registerinhalte
	pop	XL	
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet 10min (Wait 10min)
;	Der Fehler durch zusaetz. Befehle kann hier vernachlässigt werden. 
;-------------------------------------------------------------------------------

W10min:	push	XL	;rette verwendete Registerinhalte
	push	XH
	ldi	XL,0x60	;Zaehler = 60000d
	ldi	XH,0xEA	;
W10minL: rcall	W10ms	;Rufe innere Schleife (Unterprogramm W10ms)
	sbiw	XL,1	;Dekrementiere den Schleifenzaehler X
	brne	W10minL	;Verlasse die Schleife bei Schleifenzaehler X=0
	pop	XH	;Wiederherstellung der verwendeten Registerinhalte
	pop	XL	
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet X mal 1us (Wait X*1us)
;       Der Anfangswert von X muss im Hauptprogramm gesetzt werden!
;       Der Fehler durch die zusaetzlichen Befehle kann oft vernachlässigt
;       werden
;-------------------------------------------------------------------------------

WX1us:  push    XL      ;rette verwendete Registerinhalte
        push    XH
WX1usL: rcall   W1us    ;Rufe innere Schleife (Unterprogramm W100us)
        sbiw    XL,1    ;Dekrementiere den Schleifenzaehler X
        brne    WX1usL  ;Verlasse die Schleife bei Schleifenzaehler X=0
        pop     XH      ;Wiederherstellung der verwendeten Registerinhalte
        pop     XL
        ret             ;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet X mal 10us (Wait X*10us)
;       Der Anfangswert von X muss im Hauptprogramm gesetzt werden!
;       Der Fehler durch die zusaetzlichen Befehle kann oft vernachlässigt
;       werden
;-------------------------------------------------------------------------------

WX10us: push    XL      ;rette verwendete Registerinhalte
        push    XH
WX10usL: rcall   W10us  ;Rufe innere Schleife (Unterprogramm W100us)
        sbiw    XL,1    ;Dekrementiere den Schleifenzaehler X
        brne    WX10usL ;Verlasse die Schleife bei Schleifenzaehler X=0
        pop     XH      ;Wiederherstellung der verwendeten Registerinhalte
        pop     XL
        ret             ;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet X mal 100us (Wait X*100us)
;	Der Anfangswert von X muss im Hauptprogramm gesetzt werden!
;       Der Fehler durch die zusaetzlichen Befehle kann oft vernachlässigt
;       werden (max Fehler bei einmaligem Aufruf < 1,2%).
;-------------------------------------------------------------------------------

WX100us: push	XL	;rette verwendete Registerinhalte
	push	XH
WX100usL: rcall	W100us	;Rufe innere Schleife (Unterprogramm W100us)
	sbiw	XL,1	;Dekrementiere den Schleifenzaehler X
	brne	WX100usL ;Verlasse die Schleife bei Schleifenzaehler X=0
	pop	XH	;Wiederherstellung der verwendeten Registerinhalte
	pop	XL	
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet X mal 1ms (Wait X*1ms)
;	Der Anfangswert von X muss im Hauptprogramm gesetzt werden!
;       Der Fehler durch die zusaetzlichen Befehle kann meist vernachlässigt
;       werden (max Fehler bei einmaligem Aufruf < 0,12%).
;-------------------------------------------------------------------------------

WX1ms:	tst     XL       ;Falls X = 0 schnellstmoeglich zurueck
        brne    WX1msp
        tst     XH
        brne    WX1msp
        ret
WX1msp: push	XL	;rette verwendete Registerinhalte
	push	XH
WX1msL:	rcall	W1ms	;Rufe innere Schleife (Unterprogramm W1ms)
	sbiw	XL,1	;Dekrementiere den Schleifenzaehler X
	brne	WX1msL	;Verlasse die Schleife bei Schleifenzaehler X=0
	pop	XH	;Wiederherstellung der verwendeten Registerinhalte
	pop	XL	
	ret		;Ruecksprung ins Hauptprogramm
;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet X mal 10ms (Wait X*10ms)
;	Der Anfangswert von X muss im Hauptprogramm gesetzt werden!
;	Der Fehler durch die push- und pop-Befehle kann hier vernachlässigt
;	werden. 
;-------------------------------------------------------------------------------

WX10ms:	push	XL	;rette verwendete Registerinhalte
	push	XH
WX10msL: rcall	W10ms	;Rufe innere Schleife (Unterprogramm W10ms)
	sbiw	XL,1	;Dekrementiere den Schleifenzaehler X
	brne	WX10msL	;Verlasse die Schleife bei Schleifenzaehler X=0
	pop	XH	;Wiederherstellung der verwendeten Registerinhalte
	pop	XL	
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet X mal 100ms (Wait X*100ms)
;	Der Anfangswert von X muss im Hauptprogramm gesetzt werden!
;	Der Fehler durch die push- und pop-Befehle kann hier vernachlässigt
;	werden. 
;-------------------------------------------------------------------------------

WX100ms: push	XL	;rette verwendete Registerinhalte
	push	XH
WX100msL: rcall	W100ms	;Rufe innere Schleife (Unterprogramm W100ms)
	sbiw	XL,1	;Dekrementiere den Schleifenzaehler X
	brne	WX100msL ;Verlasse die Schleife bei Schleifenzaehler X=0
	pop	XH	;Wiederherstellung der verwendeten Registerinhalte
	pop	XL	
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet X mal 1s (Wait X*1s)
;	Der Anfangswert von X muss im Hauptprogramm gesetzt werden!
;	Der Fehler durch die push- und pop-Befehle kann hier vernachlässigt
;	werden. 
;-------------------------------------------------------------------------------

WX1s:	push	XL	;rette verwendete Registerinhalte
	push	XH
WX1sL:	rcall	W1s	;Rufe innere Schleife (Unterprogramm W1s)
	sbiw	XL,1	;Dekrementiere den Schleifenzaehler X
	brne	WX1sL	;Verlasse die Schleife bei Schleifenzaehler X=0
	pop	XH	;Wiederherstellung der verwendeten Registerinhalte
	pop	XL	
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
;       Verschachtelte 16-Bit-Zeitschleife wartet X mal 1min (Wait X*1min)
;	Der Anfangswert von X muss im Hauptprogramm gesetzt werden!
;	Der Fehler durch die push- und pop-Befehle kann hier vernachlässigt
;	werden. 
;-------------------------------------------------------------------------------

WX1min:	push	XL	;rette verwendete Registerinhalte
	push	XH
WX1minL: rcall	W1min	;Rufe innere Schleife (Unterprogramm W1min)
	sbiw	XL,1	;Dekrementiere den Schleifenzaehler X
	brne	WX1minL	;Verlasse die Schleife bei Schleifenzaehler X=0
	pop	XH	;Wiederherstellung der verwendeten Registerinhalte
	pop	XL	
	ret		;Ruecksprung ins Hauptprogramm

;-------------------------------------------------------------------------------
